# Generated by roxygen2: do not edit by hand

S3method(as.data.frame,wf_quality_report)
S3method(as.data.frame,wf_variance_result)
S3method(generics::augment,wf_weights)
S3method(generics::glance,wf_blend_result)
S3method(generics::glance,wf_diagnostics)
S3method(generics::glance,wf_variance_result)
S3method(generics::glance,wf_weights)
S3method(generics::tidy,wf_blend_result)
S3method(generics::tidy,wf_diagnostics)
S3method(generics::tidy,wf_variance_result)
S3method(generics::tidy,wf_weights)
S3method(plot,wf_auto_trim)
S3method(plot,wf_blend_result)
S3method(plot,wf_diagnostics)
S3method(plot,wf_propensity_weights)
S3method(plot,wf_weights)
S3method(print,wf_attrition_weights)
S3method(print,wf_auto_trim)
S3method(print,wf_autoweigh_result)
S3method(print,wf_blend_result)
S3method(print,wf_cell_merge_plan)
S3method(print,wf_collapse_plan)
S3method(print,wf_design_data)
S3method(print,wf_diagnostics)
S3method(print,wf_impact)
S3method(print,wf_influence)
S3method(print,wf_ladder_draft)
S3method(print,wf_pipeline)
S3method(print,wf_poststrat_plan)
S3method(print,wf_precheck)
S3method(print,wf_quality_report)
S3method(print,wf_replicate_weights)
S3method(print,wf_validation)
S3method(print,wf_variance_result)
S3method(print,wf_weight_plan)
S3method(print,wf_weights)
S3method(summary,wf_poststrat_plan)
export(as_svrepdesign)
export(as_svydesign)
export(wf_apply_collapse)
export(wf_approve_plan)
export(wf_assess_impact)
export(wf_attach_weights)
export(wf_attrition)
export(wf_audit_export)
export(wf_auto_trim)
export(wf_autoweigh)
export(wf_blend)
export(wf_calibrate)
export(wf_collapse_ladder)
export(wf_compose)
export(wf_diagnose)
export(wf_dims)
export(wf_execute_plan)
export(wf_guided_execute)
export(wf_guided_plan)
export(wf_import_reference)
export(wf_import_target)
export(wf_influence)
export(wf_pipeline)
export(wf_plan_cells)
export(wf_plan_poststrat)
export(wf_plan_weights)
export(wf_poststrat)
export(wf_precheck)
export(wf_prepare_design)
export(wf_propensity)
export(wf_rake)
export(wf_replicates)
export(wf_report)
export(wf_run)
export(wf_suggest_collapse)
export(wf_suggest_ladder)
export(wf_target_population)
export(wf_target_propensity)
export(wf_target_reference)
export(wf_target_template)
export(wf_validate)
export(wf_variance)
