## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(collapse = TRUE, comment = "#>")

## ----influence, eval=FALSE----------------------------------------------------
# influence <- wf_influence(
#   locked,
#   target = target,
#   sample = design$data,
#   id = design$roles$id,
#   top = 20
# )
# 
# influence$table

## ----impact, eval=FALSE-------------------------------------------------------
# impact <- wf_assess_impact(
#   locked,
#   analysis_data,
#   id = "person_id",
#   outcomes = c("score", "approved")
# )
# 
# wf_report(impact, audience = "statistician")

