## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(collapse = TRUE, comment = "#>")

## ----conversion, eval=FALSE---------------------------------------------------
# analysis_ready <- wf_attach_weights(
#   analysis_data,
#   locked,
#   id = "person_id",
#   weight_name = ".weight"
# )
# 
# design_survey <- as_svydesign(
#   locked,
#   analysis_ready,
#   id = "person_id"
# )

