## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(collapse = TRUE, comment = "#>")

## ----template, eval=FALSE-----------------------------------------------------
# dims <- wf_dims(
#   age_group = c("18-34", "35-54", "55+"),
#   region = c("north", "south")
# )
# 
# wf_target_template("population-margins.csv", dims)

## ----import, eval=FALSE-------------------------------------------------------
# target <- wf_import_target(
#   "population-margins.csv",
#   "population-margins.csv.source.dcf",
#   dims,
#   key_map = c(age_group = "age_group", region = "region"),
#   count = "population_count",
#   production = TRUE
# )

