## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(collapse = TRUE, comment = "#>")

## ----precheck, eval=FALSE-----------------------------------------------------
# precheck <- wf_precheck(
#   design$data,
#   target,
#   id = design$roles$id
# )
# 
# precheck$issues

## ----execute, eval=FALSE------------------------------------------------------
# plan <- wf_plan_weights(design, target, dims, method = "raking")
# approval <- wf_approve_plan(
#   plan,
#   approver = "Reviewer full name",
#   role = "Statistician"
# )
# locked <- wf_execute_plan(plan, approval, design, target)
# 
# diagnostics <- wf_diagnose(locked, target = target)

## ----direct, eval=FALSE-------------------------------------------------------
# via_dispatcher <- wf_calibrate(
#   design,
#   target,
#   method = "raking"
# )

