## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(collapse = TRUE, comment = "#>")

## ----checks, eval=FALSE-------------------------------------------------------
# design <- wf_prepare_design(
#   design_only,
#   id = "person_id",
#   calibration = c("age_group", "region"),
#   base_weight = "base_weight"
# )
# 
# target <- wf_import_target(
#   "population-margins.csv",
#   "population-margins.csv.source.dcf",
#   dims,
#   key_map = c(age_group = "age_group", region = "region"),
#   count = "population_count"
# )
# 
# precheck <- wf_precheck(design$data, target, id = design$roles$id)
# cell_plan <- wf_plan_cells(design, target, dims)

