## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(collapse = TRUE, comment = "#>")

## ----guided, eval=FALSE-------------------------------------------------------
# guided <- wf_autoweigh(
#   design,
#   target,
#   dims,
#   method = "raking",
#   trim = NULL,
#   interactive = FALSE,
#   lang = "en"
# )
# 
# guided$weights
# guided$diagnostics
# guided$ledger

## ----views, eval=FALSE--------------------------------------------------------
# wf_report(guided$weights, audience = "decision", lang = "zh_CN")
# wf_report(guided$weights, audience = "statistician", lang = "en")

