% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/report.R
\name{as.data.frame.wf_quality_report}
\alias{as.data.frame.wf_quality_report}
\title{Convert a weighting quality report to a data frame}
\usage{
\method{as.data.frame}{wf_quality_report}(x, ...)
}
\arguments{
\item{x}{A \code{wf_quality_report} object.}

\item{...}{Unused.}
}
\value{
The report's primary table.
}
\description{
Convert a weighting quality report to a data frame
}
