% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plots.R
\name{plot.wf_auto_trim}
\alias{plot.wf_auto_trim}
\title{Plot an automatic trim frontier}
\usage{
\method{plot}{wf_auto_trim}(x, lang = NULL, ...)
}
\arguments{
\item{x}{A \code{wf_auto_trim} object.}

\item{lang}{Output language.}

\item{...}{Additional arguments passed to the initial plot calls.}
}
\value{
Invisibly returns \code{x}.
}
\description{
Draws the worst design effect and residual margin error for every feasible
finite cap.
}
