% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/variance.R
\name{as.data.frame.wf_variance_result}
\alias{as.data.frame.wf_variance_result}
\title{Coerce a variance result to a data frame}
\usage{
\method{as.data.frame}{wf_variance_result}(x, ...)
}
\arguments{
\item{x}{A \code{wf_variance_result} object.}

\item{...}{Unused.}
}
\value{
The result table as a data frame.
}
\description{
Coerce a variance result to a data frame
}
