% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/weight-plan.R
\name{print.wf_weight_plan}
\alias{print.wf_weight_plan}
\title{Print a reviewable weight plan}
\usage{
\method{print}{wf_weight_plan}(x, ...)
}
\arguments{
\item{x}{A \code{wf_weight_plan} object.}

\item{...}{Reserved for future use.}
}
\value{
\code{x}, invisibly.
}
\description{
Print a reviewable weight plan
}
