% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/weight-execution.R
\name{wf_attach_weights}
\alias{wf_attach_weights}
\title{Attach locked weights to analysis data by exact unit ID}
\usage{
wf_attach_weights(data, weights, id, weight_name = ".weight")
}
\arguments{
\item{data}{Full analysis data frame.}

\item{weights}{An unchanged \code{wf_locked_weights} object.}

\item{id}{Unique identifier column in \code{data}.}

\item{weight_name}{Name of the appended weight column.}
}
\value{
\code{data} in its original row order with one locked-weight column.
}
\description{
Attach locked weights to analysis data by exact unit ID
}
