% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/weight-execution.R
\name{wf_guided_execute}
\alias{wf_guided_execute}
\title{Execute a guided workflow with an external human approval}
\usage{
wf_guided_execute(workflow, approval)
}
\arguments{
\item{workflow}{An unchanged \code{wf_safe_workflow}.}

\item{approval}{A matching \code{wf_plan_approval} created separately.}
}
\value{
A \code{wf_locked_weights} object.
}
\description{
Execute a guided workflow with an external human approval
}
