% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/collapse.R
\name{wf_apply_collapse}
\alias{wf_apply_collapse}
\title{Apply a category collapse plan}
\usage{
wf_apply_collapse(sample, target, plan)
}
\arguments{
\item{sample}{Sample data frame.}

\item{target}{A \code{wf_target} object.}

\item{plan}{List with \code{dim} and named character vector \code{map}, or a
\code{wf_collapse_plan} object from \code{wf_suggest_collapse()}.}
}
\value{
A list with collapsed \code{sample} and \code{target}.
}
\description{
Applies category-merge maps consistently to sample data and a target object.
}
