% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plots.R
\name{plot.wf_diagnostics}
\alias{plot.wf_diagnostics}
\title{Plot weight diagnostics}
\usage{
\method{plot}{wf_diagnostics}(x, lang = NULL, ...)
}
\arguments{
\item{x}{A \code{wf_diagnostics} object.}

\item{lang}{Output language.}

\item{...}{Additional arguments passed to \code{\link[graphics:dotchart]{graphics::dotchart()}}.}
}
\value{
Invisibly returns \code{x}.
}
\description{
Draws design effects and effective-sample-size shares by group.
}
